import fetch from 'node-fetch';

// Function to handle form submission and send transactional email via Listmonk
export async function submitToListmonk(req, res) {
    const { name, email, message } = req.body;

    const listmonkSubscriberUrl = process.env.LISTMONK_URL;
    const listmonkUsername = process.env.LISTMONK_USERNAME;
    const listmonkPassword = process.env.LISTMONK_PASSWORD;
    const listmonkBaseUrl = process.env.LISTMONK_BASE_URL;
    const listId = parseInt(process.env.LIST_ID, 10);

    // Encode username and password as base64
    const auth = Buffer.from(`${listmonkUsername}:${listmonkPassword}`).toString('base64');

    console.log(`Sending request to Listmonk with URL: ${listmonkSubscriberUrl}`);

    const payload = {
        email,
        name,
        lists: [listId],
        status: "enabled",
        fields: { message }
    };

    console.log('Sending subscriber to Listmonk with payload:', payload);

    async function fetchWithRetry(url, options, retries = 3) {
        for (let i = 0; i < retries; i++) {
            console.log(`Attempt ${i + 1} sending request to ${url}`);
            try {
                const res = await fetch(url, options);
                if (!res.ok) throw new Error(`HTTP ${res.status}`);
                return res;
            } catch (err) {
                console.log(`Attempt ${i + 1} failed: ${err}`);
                if (i === retries - 1) throw err;
                await new Promise(resolve => setTimeout(resolve, 500));
            }
        }
    }

    try {
        // Add subscriber
        const response = await fetchWithRetry(listmonkSubscriberUrl, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': `Basic ${auth}`
            },
            body: JSON.stringify(payload),
        },12);

        if (!response.ok) {
            const errorBody = await response.text();
            throw new Error(`Listmonk API error: ${response.statusText}. Response body: ${errorBody}`);
        }

        const result = await response.json();
        console.log('Subscriber added successfully:', result);

        // Send transactional email
        const transactionalUrl = `${listmonkBaseUrl}/tx`;
        const txPayload = {
          template_id: parseInt(process.env.PRESET_CAMPAIGN_ID, 10),
          subscriber_emails: [ email ],
          from_email: "Janek <janek@melonion.me>",
          data: { name, message }
        };
        
        const txResponse = await fetchWithRetry(transactionalUrl, {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
            'Authorization': `Basic ${auth}`
          },
          body: JSON.stringify(txPayload)
        }, 12);
        
        if (!txResponse.ok) {
          const errorBody = await txResponse.text();
          throw new Error(`Listmonk TX error: ${txResponse.statusText}. ${errorBody}`);
        }

        const dispatchResult = await txResponse.json();
        console.log('tx dispatched successfully:', dispatchResult);

        res.status(200).json({ message: 'Subscription and welcome email sent successfully!', result });

    } catch (error) {
        console.error('Error submitting to Listmonk:', error);
        res.status(500).json({ message: 'Failed to submit to Listmonk', error: error.message });
    }
}
